
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.009.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28",
    "perl" => "5.010"
  },
  "dist_abstract" => "functions and methods with parameter lists and type constraints",
  "dist_author" => [
    "David Zurborg <zurborg\@cpan.org>"
  ],
  "dist_name" => "Moove",
  "dist_version" => "0.006",
  "license" => "open_source",
  "module_name" => "Moove",
  "recursive_test_files" => 1,
  "requires" => {
    "Carp" => 0,
    "Data::OptList" => "0.109",
    "Function::Parameters" => "1.0703",
    "Import::Into" => "1.002004",
    "Syntax::Feature::Try" => "1.003",
    "Type::Registry" => 0,
    "Type::Tiny" => "1.000006",
    "Type::Utils" => 0,
    "constant" => 0,
    "perl" => "5.014",
    "strictures" => 2,
    "syntax" => 0
  },
  "test_requires" => {
    "Test::More" => 0,
    "Test::Most" => 0,
    "Type::Library" => 0,
    "Types::Standard" => 0,
    "perl" => "5.014"
  }
);


my %fallback_build_requires = (
  "Module::Build" => "0.28",
  "Test::More" => 0,
  "Test::Most" => 0,
  "Type::Library" => 0,
  "Types::Standard" => 0,
  "perl" => "5.014"
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
